#include <tap.h>
#include "libFireBird.h"

TAP_ID( 0x8100feed );
#define TAPNAME "MHEG State Patcher V2"
TAP_PROGRAM_NAME(TAPNAME);
TAP_AUTHOR_NAME("R2-D2");
TAP_DESCRIPTION("Patches TAP_GetState to notice MHEG state");
TAP_ETCINFO(__DATE__);

#define FWBASE		((void*)0x80000000)
#define MHEGSTATEID	("Mh")

dword TAP_EventHandler( word event, dword param1, dword param2 )
{
	return param1;
}

typedef struct {
	char *code;
	char *mask;
} Signature;

Signature orig XDATA = {
	"00000000 00000000 03801025 00000000 00000000 0060e025 27a40010 0c05e771 27a50014 11400003 00000000 8fb90010 ad590000 51200004 8d990000 8fb90014 ad390000 8d990000 0019c940 0179c821 8f2c0004 00000000 00000000 03801025 00000000 00000000 0180e025 8fa20010",
	"FFFFFFFF FFFFFFFF FFFFFFFF FFFFFFFF FFFFFFFF FFFFFFFF FFFFFFFF FC000000 FFFFFFFF FFFFFFFF FFFFFFFF FFFFFFFF FFFFFFFF FFFFFFFF FFFFFFFF FFFFFFFF FFFFFFFF FFFFFFFF FFFFFFFF FFFFFFFF FFFFFFFF FFFFFFFF FFFFFFFF FFFFFFFF FFFFFFFF FFFFFFFF FFFFFFFF FFFFFFFF"
};

Signature patch XDATA = {
	"8c6e8b64 11c00003 00000000 8dce0008 8dce0034 0060e025 27a40010 0c05e771 27a50014 8fb90010 17200008 8fb80014 240b001f 170b0005 2dc10004 54200003 00000000 24190004 24180022 11400002 00000000 ad590000 11200002 00000000 ad380000 8dec0004 0180e025 03201025",
	"FFFF0000 FFFFFFFF FFFFFFFF FFFFFFFF FFFFFFFF FFFFFFFF FFFFFFFF FC000000 FFFFFFFF FFFFFFFF FFFFFFFF FFFFFFFF FFFFFFFF FFFFFFFF FFFFFFFF FFFFFFFF FFFFFFFF FFFFFFFF FFFFFFFF FFFFFFFF FFFFFFFF FFFFFFFF FFFFFFFF FFFFFFFF FFFFFFFF FFFFFFFF FFFFFFFF FFFFFFFF"
};

/*****************************************************************************/
/* TAP_main */
/*****************************************************************************/

int TAP_Main(void)
{
	word *pMHEG = NULL;
	InitTAPex();
	pMHEG = (word*)FindInstructionSequence(
		"27bdffe0 afb50014 afb60018 afbe001c afbf0010 8f9e8b64 13c00007 0080a825 0c070c1a",
		"ffffffff ffffffff ffffffff ffffffff ffffffff ffff0000 ffffffff ffffffff ff000000",
		0x80190000, 0x80200000, 5, FALSE);
	if(pMHEG == NULL)
	{
		TAP_Print("mheg_state: no MHEG variable (fw: %d:%x)\n",SysID,ApplID);
	}
	else
	{
		if(PatchIsInstalled(FWBASE, MHEGSTATEID))
		{
			word * pGetState = (word*)FindInstructionSequence(patch.code, patch.mask, 0x80140000, 0x80200000, 0, FALSE);
			if(pGetState == NULL)
			{
				TAP_Print("mheg_state: ID already installed, but unknown state\n");
			}
			else
			{
				TAP_Print("mheg_state: removing patch\n");
				PatchInstructionSequence((dword)pGetState, orig.code, orig.mask);
				PatchRemoveID(FWBASE,MHEGSTATEID);
			}
		}
		else // Not Installed
		{
			word *pGetState = (word*)FindInstructionSequence(orig.code, orig.mask, 0x80140000, 0x80200000, 0, FALSE);
			if(pGetState == NULL)
			{
				TAP_Print("mheg_state: unsupported firmware %d:%x\n",SysID,ApplID);
			}
			else
			{
				TAP_Print("mheg_state: applying patch\n");
				PatchInstructionSequence((dword)pGetState, patch.code, patch.mask);
				pGetState[1] = pMHEG[1];
				PatchInstallID(FWBASE,MHEGSTATEID);
			}
		}
	}
	return 0;
}
